 /*!****************************************************************************
 *    \file cell.h
 *    This file contains the definition of the cell class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include "twin.h"
#include "variable.h"
#include "iboolean.h"
#include "tree.h"

#ifndef CELL_HH
#define CELL_HH

////////////////////////////////////////////////////////////////////////////////

/*!*****************************************************************************
*    \class cell
*    \brief Class for the Star Algorithm.
*    This class represents a cartesian product of modal intervals.
*******************************************************************************/
class cell
{
private:
        twin Twin;                          ///< Cell Twin
        vector<variable> Xi;                ///< Improper Variables Vector
        vector<variable> Xp;                ///< Proper Variables Vector
        vector<variable>* XpStrip;          ///< Pointer to the Proper Variables Vector of the strip container
        int XiDim;                          ///< Selected Improper Dimention for bisection
        iboolean Consistent;                ///< Consistent Flag (MinMax Optimization)
        bool Monotonic;                     ///< Monotonic Flag
        bool TOptimal;                      ///< Optimality flag (false==NonOptimal; true==Optimal)
        //int Penalty;
        bool ZeroIncluded;                           ///<
        bool ZeroExcluded;                           ///<


public:

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

cell(void);
cell(vector<variable>& Xi,vector<variable>& Xp);
cell(const cell&);

////////////////////////////////////////////////////////////////////////////////
// Destructor                                                                 //
////////////////////////////////////////////////////////////////////////////////

~cell() {}

////////////////////////////////////////////////////////////////////////////////
// Overloaded Operator                                                        //
////////////////////////////////////////////////////////////////////////////////

variable& operator[] (int);
cell& operator=(const cell &Cell);

////////////////////////////////////////////////////////////////////////////////
// Acces to members                                                           //
////////////////////////////////////////////////////////////////////////////////

vector<variable>& GetXi();
vector<variable>& GetXp();
vector<variable>& GetXpStrip();
int GetDim() const;
int GetXiDim();
int GetXpDim();
twin& GetTwin();
iboolean GetConsist(void);
bool GetMono(void);
bool GetTOptimal(void);
//int GetPenalty();
bool GetZeroIncluded(void);
bool GetZeroExcluded(void);

void SetXi(vector<variable>&);
void SetXp(vector<variable>&);
void SetXpStrip(vector<variable>&);
void SetXpStrip(vector<variable>*);
void SetXiDim(int&);
void SetTwin(twin&);
void SetConsist(iboolean&);
void SetMono(bool&);
void SetTOptimal(bool);
/*
void SetPenalty(int);
void Penalize();
*/
void SetZeroIncluded(bool);
void SetZeroExcluded(bool);

////////////////////////////////////////////////////////////////////////////////
// Member Functions                                                           //
////////////////////////////////////////////////////////////////////////////////

bool IsUniInci(void);
bool IsProp(void);
bool IsImpr(void);
bool IsUniMod(void);
bool IsTotalyMono(void);
bool IsXiTotalyMono(void);
iboolean IsConsist(void);

double GetXBigWidth(void);
void AssignXiBigDim(void);
void AssignNonMonoXiBigDim(param& Pars);
double GetNonMonoXiBigWidth(void);

void ComputeTwin(tree &T,bool StripTOptim,param&);
void ComputeTwinProp(tree &T,string &Opt,bool StripTOptim,param&);
void ComputeMonos(tree &T,const string &Mode);

void AssignTreeOptim(tree &T);
void AssignXMonos(const string &Mode,tree &T);
void MakeProp(void);
void MakeImpr(void);
void Print(fstream &fout,int PrintFlag);
void Join(cell &Cell);
void Meet(cell &Cell);

friend bool operator<(const cell &C1, const cell &C2);

friend cell operator&&(const cell &x, const cell &y);
friend cell operator||(const cell &x, const cell &y);

void AdaptCellToMinMax();
void AdaptCellToMaxMin();

}; // end Class cell

void AssignCellToTree(tree &Tree, cell& Cell,bool XpStrip);
void AssignCellToConstraint(tree &Tree,cell& Cell);
void CreateCellFromTree(cell &Cell,tree &Tree);
void AssignUniIncidence(tree &Tree, cell& Cell);
void AssignUniModal(tree &Tree, cell& Cell);
void AssignXCrossDers(tree &T, cell& C);

bool IsIntersect(cell &c1,cell &c2);


#endif
