 /*!****************************************************************************
 *    \file agenda.h
 *    This file contains the definition of the agenda class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include "strip.h"
#include "model.h"
#include "box.h"

#ifndef FSTAR_HH
#define FSTAR_HH

/*!*****************************************************************************
*    \class fstar
*    \brief Main Class for the Star Algorithm.
*    This class represents a list of strips.
*******************************************************************************/
class fstar
{

private:
         list<strip> List;                      ///> Strips list.
         twin Twin;                             ///> Global twin.
         twin AccTwin;                          ///> Accumulated Twin.
         list<strip>::iterator SelectedStrip;   ///> Selected Strip to bisect.
         bool Update;                           ///> Update twin flag.
         param Pars;                            ///> Parameters.
         model Model;                           ///> Model.
         list< list<strip>::iterator> Stock;    ///> Stock List of candidate Strips (pointers)
         list< list<strip>::iterator>::iterator PStock; ///> Stock List Pointer
         list<strip> Buffer;                    ///> Buffer Strip List.
         list<cell> MinMaxList;                 ///> List of minmax candidates.
         list<cell> MaxMinList;                 ///> List of maxmin candidates.
         list<cell> MinMaxs;                     ///> Joined MinMax Cells.
         list<cell> MaxMins;                     ///> Joined MaxMin Cells.
         int Penalty;                            /// Penalization flag
         list< list<strip>::iterator > InsertedStrips; ///> List of Inserted Strips

public:

        ivalDb AuxIval;       // Squaltrack Attribute

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

fstar();
fstar(const fstar&);

 ////////////////////////////////////////////////////////////////////////////////
// Destructor                                                                 //
////////////////////////////////////////////////////////////////////////////////

~fstar() {}

////////////////////////////////////////////////////////////////////////////////
// Acces to members                                                           //
////////////////////////////////////////////////////////////////////////////////

list<strip>& GetList();
twin& GetTwin();
twin& GetAccTwin();
bool GetUpdate();
int GetSize();
list<strip>::iterator& GetSelectStrip(void);
list< list<strip>::iterator >& GetInsertedStrips(void);
param& GetPars(void);
model& GetModel(void);
list<cell>& GetMinMaxList(void);
list<cell>& GetMaxMinList(void);
list<cell>& GetMinMaxs(void);
list<cell>& GetMaxMins(void);
list<strip>& GetBuffer(void);

void SetList(list<strip>&);
void SetTwin(twin&);
void SetAccTwin(twin&);
void SetUpdate(bool);
void SetSelectStrip(list<strip>::iterator&);
void SetInsertedStrips(list< list<strip>::iterator >&);
void SetPars(param&);
void SetModel(model&);
void SetMinMaxList(list<cell>&);
void SetMaxMinList(list<cell>&);
void SetMinMaxs(list<cell>&);
void SetMaxMins(list<cell>&);
void SetBuffer(list<strip>&);

fstar& operator=(const fstar &FSTAR);

////////////////////////////////////////////////////////////////////////////////
// Members functions                                                          //
////////////////////////////////////////////////////////////////////////////////

void CreateModel(box &X,fstream &fout);
void Initialize(box&,fstream&);
void SplitXToXpXi(vector<variable> &Xp,vector<variable> &Xi);
void Run(box&,fstream& fout);
bool UpdateTwinAndSelectStrip(fstream &fout);
void AssignBisectDim(double &XpWidth,double &XiWidth);
void Bisect(fstream &fout);
void RunProj(box& X,ivalDb &Ival,fstream& fout);
void BisectStripXp(list<strip>::iterator Strip,fstream &fout);
void BisectCellXi(list<cell>::iterator Cell,bool ComputeCell,fstream &fout);
void InsertStrip(list<strip>::iterator Strip,bool Insert,fstream &fout);
void InsertCell(list<cell>::iterator Cell,bool Insert,bool ComputeCell,fstream &fout);
void PrintSolution(double &CTime);
void AccumTwin(list<strip>::iterator);
void IncrInsertStrips(void);
bool IsEliminateStrip(strip &Strip,fstream &fout);
void SelectStrip(fstream &fout);
bool IsIncludedStrip(strip &Strip,fstream &fout);
bool IsProp();
bool IsImpr();
bool IsZeroIncluded(void);
bool IsZeroExcluded(void);
bool IsToleralceReached(void);
bool PrintScreen(int Step);
void DetermineModality();
void Delete();
bool IsInclusionStop(void);
bool IsThereImprovement();
iboolean Propagate(box& X,fstream &fout,string &Mode);
bool IsStripInclusionStop(twin& Twin);
void AssignRelEps(box& X,double Percentage);
iboolean CheckPunctualConsistency(box &X,bool Mode);
void EvaluateProj(box& X,ivalDb &Ival,fstream& fout);

////////////////////////////////////////////////////////////////////////////////
// Friend functions                                                          //
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// MinMax Member functions                                                    //
////////////////////////////////////////////////////////////////////////////////
void AddListBufferToMinMaxLists(void);
void AddStripToMinMaxLists(strip &Strip);
void AddCellToMinMaxVector(cell &Cell);
void JoinMinMaxList(void);
void JoinMaxMinList(void);

void AssignXWidths(void);

}; // end Class fstar

double GetMaxCellWidth(list<cell> &List);

#endif




