 /*!****************************************************************************
 *    \file strip.h
 *    This file contains the definition of the strip class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include "cell.h"
#include <list.h>
#include "model.h"
#include "param.h"
#include "tree.h"
#include "Msgs.h"

#ifndef STRIP_HH
#define STRIP_HH

////////////////////////////////////////////////////////////////////////////////

/*!*****************************************************************************
*    \class strip
*    \brief Class for the Star Algorithm.
*    This class represents a list of cells sharing the same vector of proper variables.
*******************************************************************************/
class strip
{
private:
        twin Twin;                           ///> Inner and outer aproximations;
        vector<variable> Xp;                 ///> Proper Variables Vector;
        vector<variable> Xi;                 ///> Improper Variable Vector;
        list<cell>::iterator SelectedCell;   ///> Selected Cell to Bisect;
        int XpDim;                           ///> Seleted Proper Dimention to Bisect;
        list<cell> Cells;                    ///> Cells List (Candidate cells to be bisected);
        list<cell> WareHouse;                ///> WareHouse of Cells (Cells not to bisect);
        iboolean Consistent;                 ///> Consistency flag (Optimization mode);
        bool JoinFlag;                       ///> Flag which indicates if the Strip has to be joined with the global twin. (MinMax Optimization)
        ivalDb Excess;                       ///> Excess approximation used for constrained minmax optimization.
        bool Update;                         ///> Update Flag;
        list< list<cell>::iterator > Stock;   ///> Stock List of candidate Cells (pointers)
        list< list<cell>::iterator>::iterator PStock; ///> Stock List Pointer
        bool TOptimal;                       //> Tree optimality Flag
        //int Penalty;
        list< list<cell>::iterator >   InsertedCells;
        bool BisectXp;
        bool ZeroIncluded;                       ///>
        bool ZeroExcluded;                       ///>



public:

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

strip(void);
strip(const cell &Cell);
strip(const strip &Strip);
strip(vector<variable> &Pvars,vector<variable> &Ivars);

////////////////////////////////////////////////////////////////////////////////
// Destructor                                                                 //
////////////////////////////////////////////////////////////////////////////////

~strip() {}

////////////////////////////////////////////////////////////////////////////////
// Overloaded Operator                                                        //
////////////////////////////////////////////////////////////////////////////////

variable& operator[] (int);
strip& operator=(const strip& P);

////////////////////////////////////////////////////////////////////////////////
// Acces to members                                                           //
////////////////////////////////////////////////////////////////////////////////

twin& GetTwin();
vector<variable>&  GetXp();
vector<variable>&  GetXi();
bool GetUpdate();
list<cell>& GetCells();
list<cell>& GetWareHouse();
int GetXpDim();
int GetDim();
list<cell>::iterator& GetSelectCell();
iboolean GetConsist(void);
list< list<cell>::iterator >& GetInsertedCells();
bool GetJoinFlag();
ivalDb GetExcess(void);
list< list<cell>::iterator>& GetStock(void);
list< list<cell>::iterator>::iterator GetPStock(void);
bool GetTOptimal(void);
//int GetPenalty();
bool GetBisectXp(void);
bool GetZeroIncluded(void);
bool GetZeroExcluded(void);

void SetTwin(twin&);
void SetXp(vector<variable>&);
void SetXi(vector<variable>&);
void SetCells(list<cell>&);
void SetWareHouse(list<cell>&);
void SetXpDim(int);
void SetUpdate(bool);
void SetSelectCell(list<cell>::iterator &Cell);
void SetConsist(iboolean&);
void SetInsertedCells(list< list<cell>::iterator >&);
void SetJoinFlag(bool);
void SetExcess(ivalDb&);
void SetStock(list< list<cell>::iterator>&);
void SetPStock(list< list<cell>::iterator>::iterator);
void SetTOptimal(bool);
//void SetPenalty(int);
void SetBisectXp(bool);
void SetZeroIncluded(bool);
void SetZeroExcluded(bool);

////////////////////////////////////////////////////////////////////////////////
// Member Functions                                                           //
////////////////////////////////////////////////////////////////////////////////

bool IsTotalyMono(void);
bool IsXpTotalyMono(void);
bool IsCellsEmpty(void);
bool IsProp(void);

void AssignXpBigDim(void);
double GetNonMonoXpBigWidth(void);
void AssignNonMonoXpBigDim(param&);
void AddCellToCells(cell&);
void AddCellToWareHouse(cell&);
void AssignCell(fstream &fout,param &Pars);
void IncrInsertCells(void);
void ComputeUpdateTwin(bool Compute,fstream &fout,model &Model,param &Pars);
void ComputeMonos(tree &T);
void ResetMonos(void);
void AssignMonos(void);
void Print(fstream &fout,int PrintFlag,bool PrintCells);
bool IsEliminateCell(cell &Cell,fstream &fout,param &Pars);
void Contract(ivalDb &Out, int Dim);
bool IsIncluded(cell &Cell,fstream &fout,param &Pars);
void AssignTreeOptim(tree &T);
void MeetCell(cell &Cell);
bool IsThereImprovement(twin&);

//void Penalize();

}; // end Class strip

void CreateCellFromTree(strip &Strip,tree &Tree);
void AssignStripToTree(tree &Tree,strip& Strip);

#endif


